* 16 April 2016
* EQ DCE Competition
* Fedora Predictions 
* Benjamin M. Craig
*
* This code demonstrates my approach to modeling EQ-5D-5L valuation data
* and how I produced the predictions for the DCE competition.
* 
* To use this code, you will need access to STATA software and either
* internet access or downloads of 2 comma-separated values (CSV) files:
*    Prediction_file.csv
*    Exploratory_Data.csv
* 
* Specifically, this code estimates 1 model, Fedora, which includes a 20-
* parameter regression (one for each health problem in the EQ-5D-5L) and 
* 2 ancillary parameters (i.e., 22 parameters).
*
* Most model assumes constant proportionality: 
*
*   Value = Lifespan - Problems * Duration   
*
*  Fedora assumes:
*
*   Value = Lifespan^alpha - Problems * Duration^beta
*
* NOTE: if alpha and beta are 1, the model is constant proportional.
*
* The code is divided into 3 sections: 
*  (1) Import and recode responses into analytical data
*  (2) Estimate Fedora using analytical data
*  (3) Predict probabilities and estimate chi-square for Fedora 
*
* For additional information, please email: benjamin.craig@iahpr.org 

* SECTION 1: Import and recode responses into analytical data

local folder="http://iahpr.org/wordpress/wp-content/uploads/2016/03/"

* Exploratory data
import delimited using "`folder'/Exploratory_Data.csv", clear case(preserve) 
save exploratory, replace

* Prediction file
import delimited using "`folder'/Prediction_file.csv", clear case(preserve)
save predict, replace

* Recode the exploratory data
use exploratory, clear

* Switch A and B in each pair to be identical [left/right] 
gen switch=(H_A>H_B)
replace choice=1-choice if switch==1  
qui for X in any T H: gen D=X_A \ replace X_A=X_B if switch==1 \ /*
*/ replace X_B=D if switch==1 \ drop D

* Calculate pair probabilities
bysort p_id: egen C=mean(choice) /* pair probabilities */
bysort p_id: gen N=_N            /* pair sample sizes */
bysort p_id: keep if _n==1
drop r_id choice switch

save pairs, replace 

* Combine the blank prediction file with the pairs data
use predict, clear
gen switch=(H_A>H_B)
qui for X in any T H: gen D=X_A \ replace X_A=X_B if switch==1 \ /*
*/ replace X_B=D if switch==1 \ drop D
joinby p_id using pairs, unmatched(master)
gen analysis=_merge==3
drop switch _merge

* Recode lifespans into years
qui for X in any A B: gen X_time=substr(T_X,1,strpos(T_X," ")-1) \ /*
*/ destring X_time, replace force \ /*
*/ replace X_time=X_time/365.25 if strpos(T_X,"day")>0 \ /*
*/ replace X_time=X_time*7/365.25 if strpos(T_X,"week")>0 \ /*
*/ replace X_time=X_time/12 if strpos(T_X,"month")>0 \ /*
*/ replace X_time=0 if X_time==. 

* Separate the 5 domains of the EQ-5D-5L
qui for X in any A B: tostring H_X, replace \ /*
*/ gen X_MO=substr(H_X,1,1) \ gen X_SC=substr(H_X,2,1) \ /*
*/ gen X_UA=substr(H_X,3,1) \ gen X_PD=substr(H_X,4,1) \ /*
*/ gen X_AD=substr(H_X,5,1) \ destring X_MO-X_AD, replace 

* Incorporate the 5 attributes using their additive differences in value
gen dif_time=A_time-B_time
qui for X in any MO SC UA PD AD: /*
*/ gen X1=(B_X>1)-(A_X>1) \ /*
*/ gen X2=(B_X>2)-(A_X>2) \ /*
*/ gen X3=(B_X>3)-(A_X>3) \ /*
*/ gen X4=(B_X>4)-(A_X>4) 

* Separate the disadvantages of A from the disadvantages of B
qui for X in varlist MO1-AD4: gen A_X = X*(X>0) 
qui for X in varlist MO1-AD4: gen B_X = -X*(X<0) 

qui for X in any MO SC UA PD AD: /*
*/ gen a_X1=(A_X>1) \ /*
*/ gen a_X2=(A_X>2) \ /*
*/ gen a_X3=(A_X>3) \ /*
*/ gen a_X4=(A_X>4)  

qui for X in any MO SC UA PD AD: /*
*/ gen b_X1=(B_X>1) \ /*
*/ gen b_X2=(B_X>2) \ /*
*/ gen b_X3=(B_X>3) \ /*
*/ gen b_X4=(B_X>4)  

save analysis, replace

* SECTION 2: Estimate Fedora using analytical data

use analysis, clear

* Constrain parameters of the 4 equations (A,B,a,b) to be equal  
qui for Y in numlist 1/20 \ /*
*/ X_A in varlist A_MO1-A_AD4 \ /*
*/ X_B in varlist B_MO1-B_AD4 : /*
*/ constraint Y [eq1a]X_A=[eq1b]X_B
qui for Y in numlist 21/40 \ /*
*/ X_A in varlist a_MO1-a_AD4 \ /*
*/ X_B in varlist b_MO1-b_AD4 : /*
*/ constraint Y [eq2a]X_A=[eq2b]X_B
qui for Y in numlist 41/60 \ /*
*/ X_A in varlist A_MO1-A_AD4 \ /*
*/ X_a in varlist a_MO1-a_AD4 : /*
*/ constraint Y [eq1a]X_A=[eq2a]X_a

* Estimate Fedora with Constant Proportionality Assumption
capture program drop wls_fedora1
program wls_fedora1
args lnf xb1 xb2 zb1 zb2 
tempvar P A_V B_V w

* Disadvantages of B
qui gen double `A_V'=`xb1'*$ML_y4+($ML_y3-$ML_y4) if ($ML_y3>$ML_y4) 
qui replace    `A_V'=`xb1'*$ML_y3+`zb2'*($ML_y4-$ML_y3)  if ($ML_y4>=$ML_y3)
qui replace    `A_V'=`zb2'*$ML_y4 if ($ML_y3==0)
qui replace    `A_V'=$ML_y3 if ($ML_y4==0)

* Disadvantages of A
qui gen double `B_V'=`xb2'*$ML_y3+($ML_y4-$ML_y3) if ($ML_y4>$ML_y3) 
qui replace    `B_V'=`xb2'*$ML_y4+`zb1'*($ML_y3-$ML_y4)  if ($ML_y3>=$ML_y4)
qui replace    `B_V'=`zb1'*$ML_y3 if ($ML_y4==0)
qui replace    `B_V'=$ML_y4 if ($ML_y3==0)

qui gen double `P'=`A_V'/(`A_V'+`B_V')

* Construct theoretical weights for Weighted Least Squares Estimation
qui gen double `w'=(1/(`P'*(1-`P')))  	/* Uncertainty Weight */
qui replace `w'=(4*($ML_y2^2)/(2*$ML_y2-1)) if (`P'<=0)+(`P'>=1)		/* Berkson Weight if w==. */
qui replace `lnf' =-(($ML_y1-`P')^2 )*`w'
end

mat m0=J(1,80,0.1)
ml model lf wls_fedora1 /*
*/ (eq1a:C= A_MO1-A_AD4 , nocons)  /*
*/ (eq1b:N= B_MO1-B_AD4 , nocons)  /*
*/ (eq2a:A_time= a_MO1-a_AD4 , nocons)  /*
*/ (eq2b:B_time= b_MO1-b_AD4 , nocons)  /*
*/ [fweight=N] if analysis==1,  /*
*/ tech(nr) init(m0, copy) maximize missing  /*
*/ nopreserve search(off) iterate(50) /*
*/ difficult constraint(1/60) collinear
ml display
mat b_fedora0=e(b)

* NOTE: This is identical to the the Bradley-Terry WLS model
* in the Exploratory Analysis, except that it uses the theoretical weights.

* Estimate Fedora with theta, alpha and beta
capture program drop wls_fedora1
program wls_fedora1
args lnf xb1 xb2 zb1 zb2 alpha beta
tempvar P A_V B_V T_A T_B D_A D_B w 
qui gen double `T_A'=`alpha'*ln($ML_y3)  /* Alpha is the effect of lifespan */
qui gen double `T_B'=`alpha'*ln($ML_y4)
qui gen double `D_A'=`beta'*ln($ML_y3)   /* Beta is the effect of duration */
qui gen double `D_B'=`beta'*ln($ML_y4)

* Disadvantages of B
qui gen double `A_V'=`xb1'*exp(`D_B')+(exp(`T_A')-exp(`T_B')) if ($ML_y3>$ML_y4) 
qui replace    `A_V'=`xb1'*exp(`D_A')+`zb2'*(exp(`D_B')-exp(`D_A'))  if ($ML_y4>=$ML_y3)
qui replace    `A_V'=`zb2'*exp(`D_B') if ($ML_y3==0)
qui replace    `A_V'=exp(`T_A') if ($ML_y4==0)

* Disadvantages of A
qui gen double `B_V'=`xb2'*exp(`D_A')+(exp(`T_B')-exp(`T_A')) if ($ML_y4>$ML_y3) 
qui replace    `B_V'=`xb2'*exp(`D_B')+`zb1'*(exp(`D_A')-exp(`D_B'))  if ($ML_y3>=$ML_y4)
qui replace    `B_V'=`zb1'*exp(`D_A') if ($ML_y4==0)
qui replace    `B_V'=exp(`T_B') if ($ML_y3==0)

qui gen double `P'=`A_V'/(`A_V'+`B_V')

* Construct theoretical weights for Weighted Least Squares Estimation
qui gen double `w'=(1/(`P'*(1-`P')))  	/* Uncertainty Weight */
qui replace `w'=(4*($ML_y2^2)/(2*$ML_y2-1)) if (`P'<=0)+(`P'>=1)		/* Berkson Weight if w==. */
qui replace `lnf' =-(($ML_y1-`P')^2 )*`w'
end

mat m1=b_fedora0,0.5,0.5
ml model lf wls_fedora1 /*
*/ (eq1a:C= A_MO1-A_AD4 , nocons)  /*
*/ (eq1b:N= B_MO1-B_AD4 , nocons)  /*
*/ (eq2a:A_time= a_MO1-a_AD4 , nocons)  /*
*/ (eq2b:B_time= b_MO1-b_AD4 , nocons)  /*
*/ /alpha /beta [fweight=N] if analysis==1,  /*
*/ tech(nr) init(m1, copy) maximize missing  /*
*/ nopreserve search(off) iterate(50) /*
*/ difficult constraint(1/60) collinear
ml display
mat b_fedora1=e(b)

* Review the parameters of the two models
mat results=(b_fedora0[1,1..20],1,1)',(b_fedora1[1,1..20],b_fedora1[1,81..82])'
mat colnames results = CP Fedora
mat list results

* NOTE: The inclusion of 2 parameters (alpha and beta) relax the constant
* proportionality assumption and reduce the chi square by over half (3166).

* SECTION 3: Predict probabilities and estimate chi-square for Fedora

use analysis, clear

* Predict the probabilities for the Fedora model (theta, alpha, beta)
gen A_f1=0
qui for X in varlist A_MO1-A_AD4 \ Y in numlist 1/20: /*
*/ replace A_f1=A_f1+X*b_fedora1[1,Y]
gen B_f1=0
qui for X in varlist B_MO1-B_AD4  \ Y in numlist 1/20: /*
*/ replace B_f1=B_f1+X*b_fedora1[1,Y]
gen a_f1=0
qui for X in varlist a_MO1-a_AD4 \ Y in numlist 1/20: /*
*/ replace a_f1=a_f1+X*b_fedora1[1,Y]
gen b_f1=0
qui for X in varlist b_MO1-b_AD4  \ Y in numlist 1/20: /*
*/ replace b_f1=b_f1+X*b_fedora1[1,Y]

qui gen double t_A=A_time^b_fedora1[1,81] /* Alpha is the effect of lifespan */
qui gen double t_B=B_time^b_fedora1[1,81]
qui gen double D_A=A_time^b_fedora1[1,82] /* Beta is the effect of duration */
qui gen double D_B=B_time^b_fedora1[1,82]

* Disadvantages of B
qui gen double A_V=A_f1*D_B+(t_A-t_B) if (A_time>B_time) 
qui replace    A_V=A_f1*D_A+b_f1*(D_B-D_A)  if (B_time>=A_time)
qui replace    A_V=b_f1*D_B if (A_time==0)
qui replace    A_V=t_A if (B_time==0)

* Disadvantages of A
qui gen double B_V=B_f1*D_A+(t_B-t_A) if (B_time>A_time) 
qui replace    B_V=B_f1*D_B+a_f1*(D_A-D_B)  if (A_time>=B_time)
qui replace    B_V=a_f1*D_A if (B_time==0)
qui replace    B_V=t_B if (A_time==0)

qui gen double P_f1=A_V/(A_V+B_V)
replace P_f1=round(P_f1*1000)/1000 
replace P_f1=0 if P_f1<0  /* If the prediction is less than 0 */
replace P_f1=1 if P_f1>1  /* If the prediction is greater than 1 */

rename P_f1 P_fedora
save prediction, replace

* Audit predictions and compute chi square
use prediction, clear
keep if analysis==1
drop A_MO-b_AD4

gen success=round(N*C,1)
gen pv=0
local i=1 
qui while `i'<=_N {
  capture bitesti N[`i'] success[`i'] P_fedora[`i'] 
  if _rc==0 replace pv=round(r(p),0.001) if _n==`i'
  local i=`i'+1
}
replace pv=. if analysis==0
gen reject=(pv<0.01)*(analysis==1)
tab reject
twoway (scatter C P_fedora if reject==0) /*
*/     (scatter C P_fedora if reject==1), legend(off)

* NOTE: It appears that 65 out of the 1560 predictions (4%; red) are rejected
* at a p-value 0.01, which is acceptable for this competition. There is 
* certainly room for improvement; however, this would likely require 
* additional parameters beyond theta, alpha and beta. 

 * Construct empirical weights for Weighted Least Squares Estimation
gen double w=(1/(C*(1-C)))  			/* Uncertainty Weight */
replace w=(4*(N^2)/(2*N-1)) if w==.		/* Berkson Weight if w==. */

gen fedora_sq=N*(C-P_fedora)^2*w
tabstat fedora_sq, s(sum)

* NOTE: The Bradley-Terry model (shown in the Exploratory Analysis code) 
* achieved a chi square of 8243, which was much lower than its closest 
* competitor, logit WLS (21521). By adding just 2 parameters, Fedora 
* reduces the chi square to 3568. Additional parameters may improve its 
* fit further; however, these predictions seem sufficient for my entry. 
* All are welcome to build from this code with proper citation.

* Export Prediction File for Fedora
use prediction, clear
export delimited p_id T_A T_B H_A H_B P_fedora /*
*/ using "C:\data\crhe16\csv\Fedora_Predictions.csv", replace 

